# coding: utf-8

from django.test import TestCase
from django.utils.unittest import SkipTest

from kinder.core.unit.models import UnitKind

from gisdo.models import ReportFormRow, ReportForm
from gisdo.reports import ReportFormReport
from gisdo.constants import AGE_CATEGORIES_FULL, AGE_CATEGORIES_EIGHT

get_name = lambda name, key: '%s_%s' % (
    name, key.replace('-', '_').replace('.', '_'))

class CheckListsTestCase(TestCase):
    """Запускаем проверки на последнем собранном ФО, для всех доу
    """
    def setUp(self):
        try:
            self.report_form = ReportForm.objects.filter(
                in_progress=False).latest('id')
            self.report_form_row_query = ReportFormRow.objects.filter(
                report=self.report_form, unit__kind_id=UnitKind.DOU)
        except ReportForm.DoesNotExist:
            raise SkipTest('not found report')

    def _sum(self, name, data, age_cat=AGE_CATEGORIES_FULL):
        result = 0
        for key in age_cat.keys():
            result += data[get_name(name, key)]
        return result

    def test_checklist(self):
        """
        запускаем проверки на одном из мо
        :return:
        """
        if not hasattr(self, 'report_form'):
            raise SkipTest('not found report')

        print u'Найден отчет %s ' % self.report_form.presentation
        for report_form_row in self.report_form_row_query:
            print u'запускаем тесты по %s' % report_form_row.unit

            mo = report_form_row.unit.get_mo()
            app_result = ReportFormReport._collect_data(
                report_form_row, index_group=ReportFormReport.APPLICATION)
            queueu_result = ReportFormReport._collect_data(
                report_form_row, index_group=ReportFormReport.QUEUE)
            enrolled_result = ReportFormReport._collect_data(
                report_form_row, index_group=ReportFormReport.ENROLLED)
            capacities_result = ReportFormReport._collect_data(
                report_form_row, index_group=ReportFormReport.CAPACITIES)

            # проверки на суммы по возастным категориям
            self.assertTrue(
                self._sum('1_1', app_result) >= app_result['1_ALL'],
                u'Не выполнена проверка 1.1. >= 1 для %s' % mo
            )
            self.assertTrue(
                (self._sum('7_1', queueu_result) ==
                 self._sum('7', queueu_result)),
                u'Не выполнена проверка 7.1.=7 для %s' % mo
            )
            self.assertTrue(
                (self._sum('7_5', queueu_result) ==
                 self._sum('7_6', queueu_result)),
                u'Не выполнена проверка 7.6. = 7.5 для %s' % mo
            )
            sum_9_1 = self._sum('9_1', queueu_result)
            self.assertTrue(
                (sum_9_1 ==
                 queueu_result['10_ALL'] +
                 queueu_result['11_ALL']),
                u'Не выполнена проверка 9.1 = 10 + 11 для %s' % mo
            )
            self.assertTrue(
                sum_9_1 == self._sum('9', queueu_result),
                u'Не выполнена проверка 9.1.= 9 для %s' % mo
            )
            self.assertTrue(
                (self._sum('18', queueu_result) ==
                 self._sum('18_1', queueu_result) ==
                 self._sum('18_2', queueu_result)),
                u'Не выполнена проверка 18 = 18.1.= 18.2 для %s' % mo
            )
            sum_29 = enrolled_result['29_ALL']
            sum_19_1 = self._sum('19_1', enrolled_result)
            self.assertTrue(
                (self._sum('19', enrolled_result) == sum_19_1 ==
                 self._sum('19_2', enrolled_result)),
                u'Не выполнена проверка 19=19.1.=19.2 для %s' % mo
            )
            self.assertTrue(
                sum_29 <= sum_19_1,
                u'Не выполнена проверка 29 <= 19.1 для %s' % mo
            )

            # в разрезе
            for key in AGE_CATEGORIES_FULL.keys():
                age_7_2 = queueu_result[get_name('7_2', key)]
                age_18_1 = queueu_result[get_name('18_1', key)]
                age_19_1 = enrolled_result[get_name('19_1', key)]
                self.assertEqual(
                    app_result[get_name('1_1', key)],
                    (app_result[get_name('2_1', key)] +
                     app_result[get_name('3_1', key)]),
                    u'Не выполнена проверка 1.1 = 2.1+3.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (app_result[get_name('4', key)] >=
                     app_result[get_name('4_1', key)] +
                     app_result[get_name('4_2', key)]),
                    u'Не выполнена проверка 4 >= 4.1+4.2 '
                    u'для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (app_result[get_name('6', key)] <=
                     app_result[get_name('1_1', key)]),
                    u'Не выполнена проверка 6 <= 1.1 '
                    u'для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('7_1', key)] >=
                    (age_7_2 +
                     queueu_result[get_name('7_3', key)]),
                    u'Не выполнена проверка 7.1. >= 7.2. + 7.3'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('7_4', key)] <=
                    age_7_2,
                    u'Не выполнена проверка 7.4 <=7.2'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('7_5', key)] >=
                    (age_18_1 +
                     age_7_2 -
                     queueu_result[get_name('7_4', key)]),
                    u'Не выполнена проверка 7.5. >= 18.1.+7.2.-7.4'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('9', key)] <=
                    queueu_result[get_name('18', key)],
                    u'Не выполнена проверка 9<=18'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('9_1', key)] <=
                    age_18_1,
                    u'Не выполнена проверка 9.1<=18.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (queueu_result[get_name('12', key)] <=
                    age_18_1 +
                     enrolled_result[get_name('20_2', key)]),
                    u'Не выполнена проверка 12 <= 18.1 + 20.2.'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (queueu_result[get_name('13', key)] <=
                     queueu_result[get_name('12', key)]),
                    u'Не выполнена проверка 13 <= 12'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (queueu_result[get_name('13_1', key)] <=
                     queueu_result[get_name('18_3', key)]),
                    u'Не выполнена проверка 13.1 <= 18.3'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (queueu_result[get_name('14', key)] <=
                     age_18_1 +
                     enrolled_result[get_name('20', key)]),
                    u'Не выполнена проверка 14 <= 18.1 + 20'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('15', key)] <=
                    age_18_1,
                    u'Не выполнена проверка 15 <= 18.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('16', key)] <=
                    age_18_1,
                    u'Не выполнена проверка 16 <= 18.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('17', key)] <=
                    age_18_1,
                    u'Не выполнена проверка 17 <= 18.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    queueu_result[get_name('17_1', key)] <=
                    age_18_1,
                    u'Не выполнена проверка 17.1 <= 18.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    age_19_1 >=
                    enrolled_result[get_name('21', key)] +
                    enrolled_result[get_name('22', key)] +
                    enrolled_result[get_name('24', key)],
                    u'Не выполнена проверка 19.1 >= 21 + 22 + 24'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    age_19_1 >=
                    enrolled_result[get_name('26', key)] +
                    enrolled_result[get_name('27', key)],
                    u'Не выполнена проверка 19.1. >= 26 + 27'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    age_19_1 >=
                    enrolled_result[get_name('30', key)] +
                    enrolled_result[get_name('30_1', key)] +
                    enrolled_result[get_name('30_2', key)],
                    u'Не выполнена проверка 19.1.>= 30 + 30.1 + 30.2'
                    u' для %s в ВК %s' % (mo, key)
                )

                self.assertTrue(
                    enrolled_result[get_name('20_1', key)] <=
                    enrolled_result[get_name('19_3', key)],
                    u'Не выполнена проверка 20.1 <= 19.3'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('20_2', key)] <=
                    enrolled_result[get_name('20', key)],
                    u'Не выполнена проверка 20.2. <= 20'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('21', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 21 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('22', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 22 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('22_1', key)] <=
                    enrolled_result[get_name('23', key)],
                    u'Не выполнена проверка 22.1 <= 23'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('23', key)] <=
                    enrolled_result[get_name('22', key)],
                    u'Не выполнена проверка 23 <= 22'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('24', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 24 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('25', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 25 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('26', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 26 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('27', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 27 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('28', key)] >=
                    enrolled_result[get_name('27', key)],
                    u'Не выполнена проверка 28 >= 27'
                    u' для %s в ВК %s' % (mo, key)
                )

                self.assertTrue(
                    enrolled_result[get_name('29_1', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 29.1. <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('30', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 30 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('30_1', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 30.1 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    enrolled_result[get_name('30_2', key)] <=
                    age_19_1,
                    u'Не выполнена проверка 30.2 <= 19.1'
                    u' для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    (enrolled_result[get_name('30_1', key)] +
                     enrolled_result[get_name('30_2', key)]) >= (
                            enrolled_result[get_name('22_1', key)] +
                            enrolled_result[get_name('22_2', key)] +
                            enrolled_result[get_name('22_3', key)] +
                            enrolled_result[get_name('22_4', key)] +
                            enrolled_result[get_name('22_5', key)] +
                            enrolled_result[get_name('22_6', key)] +
                            enrolled_result[get_name('22_7', key)] +
                            enrolled_result[get_name('22_8', key)]),
                    u'Не выполнена проверка 30.1.+ 30.2. >= '
                    u'22.1.+22.2.+22.3.+22.4.+22.5.'
                    u'+22.6.+22.7.+22.8 для %s' % mo
                )

            # в разрезе
            for key in AGE_CATEGORIES_EIGHT.keys():
                age_31 = capacities_result[get_name('31', key)]
                age_31_1 = capacities_result[get_name('31_1', key)]
                age_31_2 = capacities_result[get_name('31_2', key)]
                age_31_3 = capacities_result[get_name('31_3', key)]
                age_31_4 = capacities_result[get_name('31_4', key)]
                self.assertTrue(
                    age_31 >= age_31_1 + age_31_3,
                    u'Не выполнена проверка 31 > = 31.1. + 31.3 '
                    u'для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    age_31_2 <= age_31_1,
                    u'Не выполнена проверка 31.2 <= 31.1 '
                    u'для %s в ВК %s' % (mo, key)
                )
                self.assertTrue(
                    age_31_4 <= age_31,
                    u'Не выполнена проверка 31.4 <= 31 '
                    u'для %s в ВК %s' % (mo, key)
                )